""" openconfig_channel_monitor 

This model describes operational state data for an optical
channel monitor (OCM) for optical transport line system
elements such as wavelength routers (ROADMs) and amplifiers.

"""
from collections import OrderedDict

from ydk.types import Entity as _Entity_
from ydk.types import EntityPath, Identity, Enum, YType, YLeaf, YLeafList, YList, LeafDataList, Bits, Empty, Decimal64
from ydk.types import Entity, EntityPath, Identity, Enum, YType, YLeaf, YLeafList, YList, LeafDataList, Bits, Empty, Decimal64
from ydk.filters import YFilter
from ydk.errors import YError, YModelError
from ydk.errors.error_handler import handle_type_error as _handle_type_error


class ChannelMonitors(_Entity_):
    """
    Top\-level container for optical channel monitors
    
    .. attribute:: channel_monitor
    
        List of channel monitors, keyed by channel monitor name
    
        **type**: list of    :py:class:`ChannelMonitor<ydk.models.openconfig.openconfig_channel_monitor.ChannelMonitors.ChannelMonitor>`
    
    """
    _prefix = 'oc-chan-monitor'
    _revision = '2018-11-21'

    def __init__(self):
        super().__init__()
        self._top_entity = None

        self.yang_name = "channel-monitors"
        self.yang_parent_name = "openconfig-channel-monitor"
        self.is_top_level_class = True
        self.has_list_ancestor = False
        self.ylist_key_names = []
        self._child_classes = OrderedDict([("channel-monitor", ("channel_monitor", ChannelMonitors.ChannelMonitor))])
        self._leafs = OrderedDict()

        self.channel_monitor = YList(self)
        self._segment_path = lambda: "openconfig-channel-monitor:channel-monitors"
        self._is_frozen = True

    def __setattr__(self, name, value):
        self._perform_setattr(ChannelMonitors, [], name, value)

    class ChannelMonitor(_Entity_):
        """
        List of channel monitors, keyed by channel monitor name.
        
        .. attribute:: name  (key)
        
            References the optical channel monitor name
        
            **type**: str
        
            **refers to**:  :py:class:`name<ydk.models.openconfig.openconfig_channel_monitor.ChannelMonitors.ChannelMonitor.Config>`
        
        .. attribute:: config
        
            Configuration data 
        
            **type**:  :py:class:`Config<ydk.models.openconfig.openconfig_channel_monitor.ChannelMonitors.ChannelMonitor.Config>`
        
        .. attribute:: state
        
            Operational state data 
        
            **type**:  :py:class:`State<ydk.models.openconfig.openconfig_channel_monitor.ChannelMonitors.ChannelMonitor.State>`
        
            **config**: False
        
        .. attribute:: channels
        
            Enclosing container for the list of values describing the power spectral density distribution
        
            **type**:  :py:class:`Channels<ydk.models.openconfig.openconfig_channel_monitor.ChannelMonitors.ChannelMonitor.Channels>`
        
        """
        _prefix = 'oc-chan-monitor'
        _revision = '2018-11-21'

        def __init__(self):
            super().__init__()

            self.yang_name = "channel-monitor"
            self.yang_parent_name = "channel-monitors"
            self.is_top_level_class = False
            self.has_list_ancestor = False
            self.ylist_key_names = ['name']
            self._child_classes = OrderedDict([("config", ("config", ChannelMonitors.ChannelMonitor.Config)), ("state", ("state", ChannelMonitors.ChannelMonitor.State)), ("channels", ("channels", ChannelMonitors.ChannelMonitor.Channels))])
            self._leafs = OrderedDict([
                ('name', (YLeaf(YType.str, 'name'), ['str'])),
            ])
            self.name = None

            self.config = ChannelMonitors.ChannelMonitor.Config()
            self.config.parent = self
            self._children_name_map["config"] = "config"

            self.state = ChannelMonitors.ChannelMonitor.State()
            self.state.parent = self
            self._children_name_map["state"] = "state"

            self.channels = ChannelMonitors.ChannelMonitor.Channels()
            self.channels.parent = self
            self._children_name_map["channels"] = "channels"
            self._segment_path = lambda: "channel-monitor" + "[name='" + str(self.name) + "']"
            self._absolute_path = lambda: "openconfig-channel-monitor:channel-monitors/%s" % self._segment_path()
            self._is_frozen = True

        def __setattr__(self, name, value):
            self._perform_setattr(ChannelMonitors.ChannelMonitor, ['name'], name, value)

        class Config(_Entity_):
            """
            Configuration data 
            
            .. attribute:: name
            
                Reference to system\-supplied name of the port on the optical channel monitor (OCM). If this port is embedded in another card (i.e. an amplifier card) the device should still define a port representing the OCM even if it is internal and not physically present on the faceplate of the card
            
                **type**: str
            
                **refers to**:  :py:class:`name<ydk.models.openconfig.openconfig_platform.Components.Component>`
            
            .. attribute:: monitor_port
            
                Reference to system\-supplied name of the port that the channel monitor is physically connected to. This port will be of type MONITOR. This port is a tap off of the monitored\-port and would be in the same card as the monitored port. If this port is embedded in another card (i.e. an amplifier card) the device should still define a port representing the monitor port if it is internal and not physically present on the faceplate of the card
            
                **type**: str
            
                **refers to**:  :py:class:`name<ydk.models.openconfig.openconfig_platform.Components.Component>`
            
            """
            _prefix = 'oc-chan-monitor'
            _revision = '2018-11-21'

            def __init__(self):
                super().__init__()

                self.yang_name = "config"
                self.yang_parent_name = "channel-monitor"
                self.is_top_level_class = False
                self.has_list_ancestor = True
                self.ylist_key_names = []
                self._child_classes = OrderedDict([])
                self._leafs = OrderedDict([
                    ('name', (YLeaf(YType.str, 'name'), ['str'])),
                    ('monitor_port', (YLeaf(YType.str, 'monitor-port'), ['str'])),
                ])
                self.name = None
                self.monitor_port = None
                self._segment_path = lambda: "config"
                self._is_frozen = True

            def __setattr__(self, name, value):
                self._perform_setattr(ChannelMonitors.ChannelMonitor.Config, ['name', 'monitor_port'], name, value)


        class State(_Entity_):
            """
            Operational state data 
            
            .. attribute:: name
            
                Reference to system\-supplied name of the port on the optical channel monitor (OCM). If this port is embedded in another card (i.e. an amplifier card) the device should still define a port representing the OCM even if it is internal and not physically present on the faceplate of the card
            
                **type**: str
            
                **refers to**:  :py:class:`name<ydk.models.openconfig.openconfig_platform.Components.Component>`
            
                **config**: False
            
            .. attribute:: monitor_port
            
                Reference to system\-supplied name of the port that the channel monitor is physically connected to. This port will be of type MONITOR. This port is a tap off of the monitored\-port and would be in the same card as the monitored port. If this port is embedded in another card (i.e. an amplifier card) the device should still define a port representing the monitor port if it is internal and not physically present on the faceplate of the card
            
                **type**: str
            
                **refers to**:  :py:class:`name<ydk.models.openconfig.openconfig_platform.Components.Component>`
            
                **config**: False
            
            """
            _prefix = 'oc-chan-monitor'
            _revision = '2018-11-21'

            def __init__(self):
                super().__init__()

                self.yang_name = "state"
                self.yang_parent_name = "channel-monitor"
                self.is_top_level_class = False
                self.has_list_ancestor = True
                self.ylist_key_names = []
                self._child_classes = OrderedDict([])
                self._leafs = OrderedDict([
                    ('name', (YLeaf(YType.str, 'name'), ['str'])),
                    ('monitor_port', (YLeaf(YType.str, 'monitor-port'), ['str'])),
                ])
                self.name = None
                self.monitor_port = None
                self._segment_path = lambda: "state"
                self._is_frozen = True

            def __setattr__(self, name, value):
                self._perform_setattr(ChannelMonitors.ChannelMonitor.State, ['name', 'monitor_port'], name, value)


        class Channels(_Entity_):
            """
            Enclosing container for the list of values describing
            the power spectral density distribution
            
            .. attribute:: channel
            
                List of tuples describing the PSD distribution
            
                **type**: list of    :py:class:`Channel<ydk.models.openconfig.openconfig_channel_monitor.ChannelMonitors.ChannelMonitor.Channels.Channel>`
            
                **config**: False
            
            """
            _prefix = 'oc-chan-monitor'
            _revision = '2018-11-21'

            def __init__(self):
                super().__init__()

                self.yang_name = "channels"
                self.yang_parent_name = "channel-monitor"
                self.is_top_level_class = False
                self.has_list_ancestor = True
                self.ylist_key_names = []
                self._child_classes = OrderedDict([("channel", ("channel", ChannelMonitors.ChannelMonitor.Channels.Channel))])
                self._leafs = OrderedDict()

                self.channel = YList(self)
                self._segment_path = lambda: "channels"
                self._is_frozen = True

            def __setattr__(self, name, value):
                self._perform_setattr(ChannelMonitors.ChannelMonitor.Channels, [], name, value)

            class Channel(_Entity_):
                """
                List of tuples describing the PSD distribution
                
                .. attribute:: lower_frequency  (key)
                
                    Reference to the list key
                
                    **type**: int
                
                        **range:** 0..18446744073709551615
                
                    **refers to**:  :py:class:`lower_frequency<ydk.models.openconfig.openconfig_channel_monitor.ChannelMonitors.ChannelMonitor.Channels.Channel.State>`
                
                    **config**: False
                
                .. attribute:: upper_frequency  (key)
                
                    Reference to the list key
                
                    **type**: int
                
                        **range:** 0..18446744073709551615
                
                    **refers to**:  :py:class:`upper_frequency<ydk.models.openconfig.openconfig_channel_monitor.ChannelMonitors.ChannelMonitor.Channels.Channel.State>`
                
                    **config**: False
                
                .. attribute:: state
                
                    Operational state data for PSD
                
                    **type**:  :py:class:`State<ydk.models.openconfig.openconfig_channel_monitor.ChannelMonitors.ChannelMonitor.Channels.Channel.State>`
                
                    **config**: False
                
                """
                _prefix = 'oc-chan-monitor'
                _revision = '2018-11-21'

                def __init__(self):
                    super().__init__()

                    self.yang_name = "channel"
                    self.yang_parent_name = "channels"
                    self.is_top_level_class = False
                    self.has_list_ancestor = True
                    self.ylist_key_names = ['lower_frequency','upper_frequency']
                    self._child_classes = OrderedDict([("state", ("state", ChannelMonitors.ChannelMonitor.Channels.Channel.State))])
                    self._leafs = OrderedDict([
                        ('lower_frequency', (YLeaf(YType.str, 'lower-frequency'), ['int'])),
                        ('upper_frequency', (YLeaf(YType.str, 'upper-frequency'), ['int'])),
                    ])
                    self.lower_frequency = None
                    self.upper_frequency = None

                    self.state = ChannelMonitors.ChannelMonitor.Channels.Channel.State()
                    self.state.parent = self
                    self._children_name_map["state"] = "state"
                    self._segment_path = lambda: "channel" + "[lower-frequency='" + str(self.lower_frequency) + "']" + "[upper-frequency='" + str(self.upper_frequency) + "']"
                    self._is_frozen = True

                def __setattr__(self, name, value):
                    self._perform_setattr(ChannelMonitors.ChannelMonitor.Channels.Channel, ['lower_frequency', 'upper_frequency'], name, value)

                class State(_Entity_):
                    """
                    Operational state data for PSD
                    
                    .. attribute:: lower_frequency
                    
                        Lower frequency of the specified PSD
                    
                        **type**: int
                    
                            **range:** 0..18446744073709551615
                    
                        **config**: False
                    
                    .. attribute:: upper_frequency
                    
                        Upper frequency of the specified PSD
                    
                        **type**: int
                    
                            **range:** 0..18446744073709551615
                    
                        **config**: False
                    
                    .. attribute:: psd
                    
                        Power spectral density expressed in nanowatts per megahertz, nW/MHz.  These units allow the value to often be greater than 1.0.  It also avoids dealing with zero values for 0dBm.  For example, a 40GHz wide channel with 0dBm power would be\:  0dBm = 1mW = 10^6nW  40GHz = 40,000MHz  0dBm/40GHz = 10^6nW/40,000MHz = 1000/40 = 25
                    
                        **type**: str
                    
                            **length:** 4..4
                    
                        **config**: False
                    
                        **units**: nW/MHz
                    
                    """
                    _prefix = 'oc-chan-monitor'
                    _revision = '2018-11-21'

                    def __init__(self):
                        super().__init__()

                        self.yang_name = "state"
                        self.yang_parent_name = "channel"
                        self.is_top_level_class = False
                        self.has_list_ancestor = True
                        self.ylist_key_names = []
                        self._child_classes = OrderedDict([])
                        self._leafs = OrderedDict([
                            ('lower_frequency', (YLeaf(YType.uint64, 'lower-frequency'), ['int'])),
                            ('upper_frequency', (YLeaf(YType.uint64, 'upper-frequency'), ['int'])),
                            ('psd', (YLeaf(YType.str, 'psd'), ['str'])),
                        ])
                        self.lower_frequency = None
                        self.upper_frequency = None
                        self.psd = None
                        self._segment_path = lambda: "state"
                        self._is_frozen = True

                    def __setattr__(self, name, value):
                        self._perform_setattr(ChannelMonitors.ChannelMonitor.Channels.Channel.State, ['lower_frequency', 'upper_frequency', 'psd'], name, value)





    def clone_ptr(self):
        self._top_entity = ChannelMonitors()
        return self._top_entity






""" openconfig_mpls_types 

General types for MPLS / TE data model

"""
from collections import OrderedDict

from ydk.types import Entity as _Entity_
from ydk.types import EntityPath, Identity, Enum, YType, YLeaf, YLeafList, YList, LeafDataList, Bits, Empty, Decimal64
from ydk.types import Entity, EntityPath, Identity, Enum, YType, YLeaf, YLeafList, YList, LeafDataList, Bits, Empty, Decimal64
from ydk.filters import YFilter
from ydk.errors import YError, YModelError
from ydk.errors.error_handler import handle_type_error as _handle_type_error


class MplsLabel(Enum):
    """
    MplsLabel (Enum Class)

    type for MPLS label value encoding

    .. data:: IPV4_EXPLICIT_NULL = 0

        valid at the bottom of the label stack,

        indicates that stack must be popped and packet forwarded

        based on IPv4 header

    .. data:: ROUTER_ALERT = 1

        allowed anywhere in the label stack except

        the bottom, local router delivers packet to the local CPU

        when this label is at the top of the stack

    .. data:: IPV6_EXPLICIT_NULL = 2

        valid at the bottom of the label stack,

        indicates that stack must be popped and packet forwarded

        based on IPv6 header

    .. data:: IMPLICIT_NULL = 3

        assigned by local LSR but not carried in

        packets

    .. data:: ENTROPY_LABEL_INDICATOR = 7

        Entropy label indicator, to allow an LSR

        to distinguish between entropy label and applicaiton

        labels RFC 6790

    .. data:: NO_LABEL = 8

        This value is utilised to indicate that the packet that

        is forwarded by the local system does not have an MPLS

        header applied to it. Typically, this is used at the

        egress of an LSP

    """

    IPV4_EXPLICIT_NULL = Enum.YLeaf(0, "IPV4_EXPLICIT_NULL")

    ROUTER_ALERT = Enum.YLeaf(1, "ROUTER_ALERT")

    IPV6_EXPLICIT_NULL = Enum.YLeaf(2, "IPV6_EXPLICIT_NULL")

    IMPLICIT_NULL = Enum.YLeaf(3, "IMPLICIT_NULL")

    ENTROPY_LABEL_INDICATOR = Enum.YLeaf(7, "ENTROPY_LABEL_INDICATOR")

    NO_LABEL = Enum.YLeaf(8, "NO_LABEL")


class TunnelType(Enum):
    """
    TunnelType (Enum Class)

    defines the tunnel type for the LSP

    .. data:: P2P = 0

        point-to-point label-switched-path

    .. data:: P2MP = 1

        point-to-multipoint label-switched-path

    .. data:: MP2MP = 2

        multipoint-to-multipoint label-switched-path

    """

    P2P = Enum.YLeaf(0, "P2P")

    P2MP = Enum.YLeaf(1, "P2MP")

    MP2MP = Enum.YLeaf(2, "MP2MP")


class PATHCOMPUTATIONMETHOD(Identity):
    """
    base identity for supported path computation
    mechanisms
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:PATH_COMPUTATION_METHOD"):
        super().__init__(ns, pref, tag)


class PATHSETUPPROTOCOL(Identity):
    """
    base identity for supported MPLS signaling
    protocols
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:PATH_SETUP_PROTOCOL"):
        super().__init__(ns, pref, tag)


class PROTECTIONTYPE(Identity):
    """
    base identity for protection type
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:PROTECTION_TYPE"):
        super().__init__(ns, pref, tag)


class LSPROLE(Identity):
    """
    Base identity for describing the role of
    label switched path at the current node
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:LSP_ROLE"):
        super().__init__(ns, pref, tag)


class TUNNELTYPE(Identity):
    """
    Base identity from which specific tunnel types are
    derived.
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:TUNNEL_TYPE"):
        super().__init__(ns, pref, tag)


class LSPOPERSTATUS(Identity):
    """
    Base identity for LSP operational status
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:LSP_OPER_STATUS"):
        super().__init__(ns, pref, tag)


class TUNNELADMINSTATUS(Identity):
    """
    Base identity for tunnel administrative status
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:TUNNEL_ADMIN_STATUS"):
        super().__init__(ns, pref, tag)


class NULLLABELTYPE(Identity):
    """
    Base identity from which specific null\-label types are
    derived.
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:NULL_LABEL_TYPE"):
        super().__init__(ns, pref, tag)


class LSPMETRICTYPE(Identity):
    """
    Base identity for types of LSP metric specification
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:LSP_METRIC_TYPE"):
        super().__init__(ns, pref, tag)


class LOCALLYCOMPUTED(PATHCOMPUTATIONMETHOD):
    """
    indicates a constrained\-path LSP in which the
    path is computed by the local LER
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:LOCALLY_COMPUTED"):
        super().__init__(ns, pref, tag)


class EXTERNALLYQUERIED(PATHCOMPUTATIONMETHOD):
    """
    Constrained\-path LSP in which the path is
    obtained by querying an external source, such as a PCE server.
    In the case that an LSP is defined to be externally queried, it may
    also have associated explicit definitions (which are provided to the
    external source to aid computation); and the path that is returned by
    the external source is not required to provide a wholly resolved
    path back to the originating system \- that is to say, some local
    computation may also be required
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:EXTERNALLY_QUERIED"):
        super().__init__(ns, pref, tag)


class EXPLICITLYDEFINED(PATHCOMPUTATIONMETHOD):
    """
    constrained\-path LSP in which the path is
    explicitly specified as a collection of strict or/and loose
    hops
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:EXPLICITLY_DEFINED"):
        super().__init__(ns, pref, tag)


class PATHSETUPRSVP(PATHSETUPPROTOCOL):
    """
    RSVP\-TE signaling protocol
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:PATH_SETUP_RSVP"):
        super().__init__(ns, pref, tag)


class PATHSETUPSR(PATHSETUPPROTOCOL):
    """
    Segment routing
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:PATH_SETUP_SR"):
        super().__init__(ns, pref, tag)


class PATHSETUPLDP(PATHSETUPPROTOCOL):
    """
    LDP \- RFC 5036
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:PATH_SETUP_LDP"):
        super().__init__(ns, pref, tag)


class UNPROTECTED(PROTECTIONTYPE):
    """
    no protection is desired
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:UNPROTECTED"):
        super().__init__(ns, pref, tag)


class LINKPROTECTIONREQUIRED(PROTECTIONTYPE):
    """
    link protection is desired
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:LINK_PROTECTION_REQUIRED"):
        super().__init__(ns, pref, tag)


class LINKNODEPROTECTIONREQUESTED(PROTECTIONTYPE):
    """
    node and link protection are both desired
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:LINK_NODE_PROTECTION_REQUESTED"):
        super().__init__(ns, pref, tag)


class INGRESS(LSPROLE):
    """
    Label switched path is an ingress (headend)
    LSP
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:INGRESS"):
        super().__init__(ns, pref, tag)


class EGRESS(LSPROLE):
    """
    Label switched path is an egress (tailend)
    LSP
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:EGRESS"):
        super().__init__(ns, pref, tag)


class TRANSIT(LSPROLE):
    """
    Label switched path is a transit LSP
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:TRANSIT"):
        super().__init__(ns, pref, tag)


class P2P(TUNNELTYPE):
    """
    TE point\-to\-point tunnel type.
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:P2P"):
        super().__init__(ns, pref, tag)


class P2MP(TUNNELTYPE):
    """
    TE point\-to\-multipoint tunnel type.
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:P2MP"):
        super().__init__(ns, pref, tag)


class DOWN(LSPOPERSTATUS):
    """
    LSP is operationally down or out of service
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:DOWN"):
        super().__init__(ns, pref, tag)


class UP(LSPOPERSTATUS):
    """
    LSP is operationally active and available
    for traffic.
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:UP"):
        super().__init__(ns, pref, tag)


class ADMINDOWN(TUNNELADMINSTATUS):
    """
    LSP is administratively down
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:ADMIN_DOWN"):
        super().__init__(ns, pref, tag)


class ADMINUP(TUNNELADMINSTATUS):
    """
    LSP is administratively up
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:ADMIN_UP"):
        super().__init__(ns, pref, tag)


class EXPLICIT(NULLLABELTYPE):
    """
    Explicit null label is used.
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:EXPLICIT"):
        super().__init__(ns, pref, tag)


class IMPLICIT(NULLLABELTYPE):
    """
    Implicit null label is used.
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:IMPLICIT"):
        super().__init__(ns, pref, tag)


class LSPMETRICRELATIVE(LSPMETRICTYPE):
    """
    The metric specified for the LSPs to which this identity refers
    is specified as a relative value to the IGP metric cost to the
    LSP's tail\-end.
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:LSP_METRIC_RELATIVE"):
        super().__init__(ns, pref, tag)


class LSPMETRICABSOLUTE(LSPMETRICTYPE):
    """
    The metric specified for the LSPs to which this identity refers
    is specified as an absolute value
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:LSP_METRIC_ABSOLUTE"):
        super().__init__(ns, pref, tag)


class LSPMETRICINHERITED(LSPMETRICTYPE):
    """
    The metric for for the LSPs to which this identity refers is
    not specified explicitly \- but rather inherited from the IGP
    cost directly
    
    """
    _prefix = 'oc-mpls-types'
    _revision = '2017-06-21'

    def __init__(self, ns="http://openconfig.net/yang/mpls-types", pref="openconfig-mpls-types", tag="openconfig-mpls-types:LSP_METRIC_INHERITED"):
        super().__init__(ns, pref, tag)






""" openconfig_if_ethernet 

Model for managing Ethernet interfaces \-\- augments the OpenConfig
model for interface configuration and state.

"""
from collections import OrderedDict

from ydk.types import Entity as _Entity_
from ydk.types import EntityPath, Identity, Enum, YType, YLeaf, YLeafList, YList, LeafDataList, Bits, Empty, Decimal64
from ydk.types import Entity, EntityPath, Identity, Enum, YType, YLeaf, YLeafList, YList, LeafDataList, Bits, Empty, Decimal64
from ydk.filters import YFilter
from ydk.errors import YError, YModelError
from ydk.errors.error_handler import handle_type_error as _handle_type_error


class ETHERNETSPEED(Identity):
    """
    base type to specify available Ethernet link
    speeds
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:ETHERNET_SPEED"):
        super().__init__(ns, pref, tag)


class SPEED10MB(ETHERNETSPEED):
    """
    10 Mbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_10MB"):
        super().__init__(ns, pref, tag)


class SPEED100MB(ETHERNETSPEED):
    """
    100 Mbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_100MB"):
        super().__init__(ns, pref, tag)


class SPEED1GB(ETHERNETSPEED):
    """
    1 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_1GB"):
        super().__init__(ns, pref, tag)


class SPEED2500MB(ETHERNETSPEED):
    """
    2.5 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_2500MB"):
        super().__init__(ns, pref, tag)


class SPEED5GB(ETHERNETSPEED):
    """
    5 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_5GB"):
        super().__init__(ns, pref, tag)


class SPEED10GB(ETHERNETSPEED):
    """
    10 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_10GB"):
        super().__init__(ns, pref, tag)


class SPEED25GB(ETHERNETSPEED):
    """
    25 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_25GB"):
        super().__init__(ns, pref, tag)


class SPEED40GB(ETHERNETSPEED):
    """
    40 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_40GB"):
        super().__init__(ns, pref, tag)


class SPEED50GB(ETHERNETSPEED):
    """
    50 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_50GB"):
        super().__init__(ns, pref, tag)


class SPEED100GB(ETHERNETSPEED):
    """
    100 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_100GB"):
        super().__init__(ns, pref, tag)


class SPEED200GB(ETHERNETSPEED):
    """
    200 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_200GB"):
        super().__init__(ns, pref, tag)


class SPEED400GB(ETHERNETSPEED):
    """
    400 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_400GB"):
        super().__init__(ns, pref, tag)


class SPEED600GB(ETHERNETSPEED):
    """
    600 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_600GB"):
        super().__init__(ns, pref, tag)


class SPEED800GB(ETHERNETSPEED):
    """
    800 Gbps Ethernet
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_800GB"):
        super().__init__(ns, pref, tag)


class SPEEDUNKNOWN(ETHERNETSPEED):
    """
    Interface speed is unknown.  Systems may report
    speed UNKNOWN when an interface is down or unpopuplated (e.g.,
    pluggable not present).
    
    """
    _prefix = 'oc-eth'
    _revision = '2020-05-06'

    def __init__(self, ns="http://openconfig.net/yang/interfaces/ethernet", pref="openconfig-if-ethernet", tag="openconfig-if-ethernet:SPEED_UNKNOWN"):
        super().__init__(ns, pref, tag)






""" openconfig_bfd 

An OpenConfig model of Bi\-Directional Forwarding Detection (BFD)
configuration and operational state.

"""
from collections import OrderedDict

from ydk.types import Entity as _Entity_
from ydk.types import EntityPath, Identity, Enum, YType, YLeaf, YLeafList, YList, LeafDataList, Bits, Empty, Decimal64
from ydk.types import Entity, EntityPath, Identity, Enum, YType, YLeaf, YLeafList, YList, LeafDataList, Bits, Empty, Decimal64
from ydk.filters import YFilter
from ydk.errors import YError, YModelError
from ydk.errors.error_handler import handle_type_error as _handle_type_error


class BfdDiagnosticCode(Enum):
    """
    BfdDiagnosticCode (Enum Class)

    Diagnostic codes defined by BFD. These typically indicate the

    reason for a change of session state.

    .. data:: NO_DIAGNOSTIC = 0

        No diagnostic code was specified, or the session has not

        changed state.

    .. data:: DETECTION_TIMEOUT = 1

        The control detection time expired: no BFD packet was

        received within the required period.

    .. data:: ECHO_FAILED = 2

        The BFD echo function failed - echo packets have not been

        received for the required period of time.

    .. data:: FORWARDING_RESET = 3

        The forwarding plane in the local system was reset - such

        that the remote system cannot rely on the forwarding state of

        the device specifying this error code.

    .. data:: PATH_DOWN = 4

        Signalling outside of BFD specified that the path underlying

        this session has failed.

    .. data:: CONCATENATED_PATH_DOWN = 5

        When a BFD session runs over a series of path segments, this

        error code indicates that a subsequent path segment (i.e.,

        one in the transmit path between the source and destination

        of the session) has failed.

    .. data:: ADMIN_DOWN = 6

        The BFD session has been administratively disabled by the

        peer.

    .. data:: REVERSE_CONCATENATED_PATH_DOWN = 7

        In the case that a BFD session is running over a series of

        path segments, this error code indicates that a path segment

        on the reverse path (i.e., in the transmit direction from the

        destination to the source of the session) has failed.

    """

    NO_DIAGNOSTIC = Enum.YLeaf(0, "NO_DIAGNOSTIC")

    DETECTION_TIMEOUT = Enum.YLeaf(1, "DETECTION_TIMEOUT")

    ECHO_FAILED = Enum.YLeaf(2, "ECHO_FAILED")

    FORWARDING_RESET = Enum.YLeaf(3, "FORWARDING_RESET")

    PATH_DOWN = Enum.YLeaf(4, "PATH_DOWN")

    CONCATENATED_PATH_DOWN = Enum.YLeaf(5, "CONCATENATED_PATH_DOWN")

    ADMIN_DOWN = Enum.YLeaf(6, "ADMIN_DOWN")

    REVERSE_CONCATENATED_PATH_DOWN = Enum.YLeaf(7, "REVERSE_CONCATENATED_PATH_DOWN")


class BfdSessionState(Enum):
    """
    BfdSessionState (Enum Class)

    The state of the BFD session according to the system referred

    to by the context of the leaf.

    .. data:: UP = 0

        The BFD session is perceived to be up by the system.

    .. data:: DOWN = 1

        The BFD session is perceived to be down by the system.

    .. data:: ADMIN_DOWN = 2

        The BFD session is administratively disabled.

    .. data:: INIT = 3

        The BFD session is perceived to be initialising by the

        system.

    """

    UP = Enum.YLeaf(0, "UP")

    DOWN = Enum.YLeaf(1, "DOWN")

    ADMIN_DOWN = Enum.YLeaf(2, "ADMIN_DOWN")

    INIT = Enum.YLeaf(3, "INIT")


class Bfd(_Entity_):
    """
    Configuration and operational state parameters for BFD.
    
    .. attribute:: interfaces
    
        Interfaces on which BFD sessions are to be enabled
    
        **type**:  :py:class:`Interfaces<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces>`
    
    """
    _prefix = 'oc-bfd'
    _revision = '2018-11-21'

    def __init__(self):
        super().__init__()
        self._top_entity = None

        self.yang_name = "bfd"
        self.yang_parent_name = "openconfig-bfd"
        self.is_top_level_class = True
        self.has_list_ancestor = False
        self.ylist_key_names = []
        self._child_classes = OrderedDict([("interfaces", ("interfaces", Bfd.Interfaces))])
        self._leafs = OrderedDict()

        self.interfaces = Bfd.Interfaces()
        self.interfaces.parent = self
        self._children_name_map["interfaces"] = "interfaces"
        self._segment_path = lambda: "openconfig-bfd:bfd"
        self._is_frozen = True

    def __setattr__(self, name, value):
        self._perform_setattr(Bfd, [], name, value)

    class Interfaces(_Entity_):
        """
        Interfaces on which BFD sessions are to be enabled.
        
        .. attribute:: interface
        
            Per\-interface configuration and state parameters for BFD
        
            **type**: list of    :py:class:`Interface<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface>`
        
        """
        _prefix = 'oc-bfd'
        _revision = '2018-11-21'

        def __init__(self):
            super().__init__()

            self.yang_name = "interfaces"
            self.yang_parent_name = "bfd"
            self.is_top_level_class = False
            self.has_list_ancestor = False
            self.ylist_key_names = []
            self._child_classes = OrderedDict([("interface", ("interface", Bfd.Interfaces.Interface))])
            self._leafs = OrderedDict()

            self.interface = YList(self)
            self._segment_path = lambda: "interfaces"
            self._absolute_path = lambda: "openconfig-bfd:bfd/%s" % self._segment_path()
            self._is_frozen = True

        def __setattr__(self, name, value):
            self._perform_setattr(Bfd.Interfaces, [], name, value)

        class Interface(_Entity_):
            """
            Per\-interface configuration and state parameters for BFD.
            
            .. attribute:: id  (key)
            
                A reference to an identifier for the interface on which BFD is enabled
            
                **type**: str
            
                **refers to**:  :py:class:`id<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.Config>`
            
            .. attribute:: config
            
                Configuration parameters for BFD on the specified interface
            
                **type**:  :py:class:`Config<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.Config>`
            
            .. attribute:: state
            
                Operational state parameters for BFD on the specified interface
            
                **type**:  :py:class:`State<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.State>`
            
                **config**: False
            
            .. attribute:: interface_ref
            
                Reference to an interface or subinterface
            
                **type**:  :py:class:`InterfaceRef<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.InterfaceRef>`
            
            .. attribute:: micro_bfd_sessions
            
                Parameters relating to micro\-BFD sessions associated with the interface
            
                **type**:  :py:class:`MicroBfdSessions<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.MicroBfdSessions>`
            
            .. attribute:: peers
            
                Parameters relating to the BFD peers which are seen over this interface
            
                **type**:  :py:class:`Peers<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.Peers>`
            
            """
            _prefix = 'oc-bfd'
            _revision = '2018-11-21'

            def __init__(self):
                super().__init__()

                self.yang_name = "interface"
                self.yang_parent_name = "interfaces"
                self.is_top_level_class = False
                self.has_list_ancestor = False
                self.ylist_key_names = ['id']
                self._child_classes = OrderedDict([("config", ("config", Bfd.Interfaces.Interface.Config)), ("state", ("state", Bfd.Interfaces.Interface.State)), ("interface-ref", ("interface_ref", Bfd.Interfaces.Interface.InterfaceRef)), ("micro-bfd-sessions", ("micro_bfd_sessions", Bfd.Interfaces.Interface.MicroBfdSessions)), ("peers", ("peers", Bfd.Interfaces.Interface.Peers))])
                self._leafs = OrderedDict([
                    ('id', (YLeaf(YType.str, 'id'), ['str'])),
                ])
                self.id = None

                self.config = Bfd.Interfaces.Interface.Config()
                self.config.parent = self
                self._children_name_map["config"] = "config"

                self.state = Bfd.Interfaces.Interface.State()
                self.state.parent = self
                self._children_name_map["state"] = "state"

                self.interface_ref = Bfd.Interfaces.Interface.InterfaceRef()
                self.interface_ref.parent = self
                self._children_name_map["interface_ref"] = "interface-ref"

                self.micro_bfd_sessions = Bfd.Interfaces.Interface.MicroBfdSessions()
                self.micro_bfd_sessions.parent = self
                self._children_name_map["micro_bfd_sessions"] = "micro-bfd-sessions"

                self.peers = Bfd.Interfaces.Interface.Peers()
                self.peers.parent = self
                self._children_name_map["peers"] = "peers"
                self._segment_path = lambda: "interface" + "[id='" + str(self.id) + "']"
                self._absolute_path = lambda: "openconfig-bfd:bfd/interfaces/%s" % self._segment_path()
                self._is_frozen = True

            def __setattr__(self, name, value):
                self._perform_setattr(Bfd.Interfaces.Interface, ['id'], name, value)

            class Config(_Entity_):
                """
                Configuration parameters for BFD on the specified
                interface.
                
                .. attribute:: id
                
                    A unique identifier for the interface
                
                    **type**: str
                
                .. attribute:: enabled
                
                    When this leaf is set to true then the BFD session is enabled on the specified interface \- if it is set to false, it is administratively disabled
                
                    **type**: bool
                
                .. attribute:: local_address
                
                    The source IP address to be used for BFD sessions over this interface
                
                    **type**: union of the below types:
                
                        **type**: str
                
                            **pattern:** ^(([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])\\.){3}([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])$
                
                        **type**: str
                
                            **pattern:** ^(([0\-9a\-fA\-F]{1,4}\:){7}[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,7}\:\|([0\-9a\-fA\-F]{1,4}\:){1,6}\:[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,5}(\:[0\-9a\-fA\-F]{1,4}){1,2}\|([0\-9a\-fA\-F]{1,4}\:){1,4}(\:[0\-9a\-fA\-F]{1,4}){1,3}\|([0\-9a\-fA\-F]{1,4}\:){1,3}(\:[0\-9a\-fA\-F]{1,4}){1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,2}(\:[0\-9a\-fA\-F]{1,4}){1,5}\|[0\-9a\-fA\-F]{1,4}\:((\:[0\-9a\-fA\-F]{1,4}){1,6})\|\:((\:[0\-9a\-fA\-F]{1,4}){1,7}\|\:))$
                
                .. attribute:: desired_minimum_tx_interval
                
                    The minimum interval between transmission of BFD control packets that the operator desires. This value is advertised to the peer, however the actual interval used is specified by taking the maximum of desired\-minimum\-tx\-interval and the value of the remote required\-minimum\-receive interval value.  This value is specified as an integer number of microseconds
                
                    **type**: int
                
                        **range:** 0..4294967295
                
                    **units**: microseconds
                
                .. attribute:: required_minimum_receive
                
                    The minimum interval between received BFD control packets that this system should support. This value is advertised to the remote peer to indicate the maximum frequency (i.e., minimum inter\-packet interval) between BFD control packets that is acceptable to the local system
                
                    **type**: int
                
                        **range:** 0..4294967295
                
                    **units**: microseconds
                
                .. attribute:: detection_multiplier
                
                    The number of packets that must be missed to declare this session as down. The detection interval for the BFD session is calculated by multiplying the value of the negotiated transmission interval by this value
                
                    **type**: int
                
                        **range:** 1..65535
                
                .. attribute:: enable_per_member_link
                
                    When this leaf is set to true \- BFD will be enabled on each member interface of the aggregated Ethernet bundle
                
                    **type**: bool
                
                    **default value**: false
                
                """
                _prefix = 'oc-bfd'
                _revision = '2018-11-21'

                def __init__(self):
                    super().__init__()

                    self.yang_name = "config"
                    self.yang_parent_name = "interface"
                    self.is_top_level_class = False
                    self.has_list_ancestor = True
                    self.ylist_key_names = []
                    self._child_classes = OrderedDict([])
                    self._leafs = OrderedDict([
                        ('id', (YLeaf(YType.str, 'id'), ['str'])),
                        ('enabled', (YLeaf(YType.boolean, 'enabled'), ['bool'])),
                        ('local_address', (YLeaf(YType.str, 'local-address'), ['str','str'])),
                        ('desired_minimum_tx_interval', (YLeaf(YType.uint32, 'desired-minimum-tx-interval'), ['int'])),
                        ('required_minimum_receive', (YLeaf(YType.uint32, 'required-minimum-receive'), ['int'])),
                        ('detection_multiplier', (YLeaf(YType.uint16, 'detection-multiplier'), ['int'])),
                        ('enable_per_member_link', (YLeaf(YType.boolean, 'enable-per-member-link'), ['bool'])),
                    ])
                    self.id = None
                    self.enabled = None
                    self.local_address = None
                    self.desired_minimum_tx_interval = None
                    self.required_minimum_receive = None
                    self.detection_multiplier = None
                    self.enable_per_member_link = None
                    self._segment_path = lambda: "config"
                    self._is_frozen = True

                def __setattr__(self, name, value):
                    self._perform_setattr(Bfd.Interfaces.Interface.Config, ['id', 'enabled', 'local_address', 'desired_minimum_tx_interval', 'required_minimum_receive', 'detection_multiplier', 'enable_per_member_link'], name, value)


            class State(_Entity_):
                """
                Operational state parameters for BFD on the specified
                interface.
                
                .. attribute:: id
                
                    A unique identifier for the interface
                
                    **type**: str
                
                    **config**: False
                
                .. attribute:: enabled
                
                    When this leaf is set to true then the BFD session is enabled on the specified interface \- if it is set to false, it is administratively disabled
                
                    **type**: bool
                
                    **config**: False
                
                .. attribute:: local_address
                
                    The source IP address to be used for BFD sessions over this interface
                
                    **type**: union of the below types:
                
                        **type**: str
                
                            **pattern:** ^(([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])\\.){3}([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])$
                
                        **type**: str
                
                            **pattern:** ^(([0\-9a\-fA\-F]{1,4}\:){7}[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,7}\:\|([0\-9a\-fA\-F]{1,4}\:){1,6}\:[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,5}(\:[0\-9a\-fA\-F]{1,4}){1,2}\|([0\-9a\-fA\-F]{1,4}\:){1,4}(\:[0\-9a\-fA\-F]{1,4}){1,3}\|([0\-9a\-fA\-F]{1,4}\:){1,3}(\:[0\-9a\-fA\-F]{1,4}){1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,2}(\:[0\-9a\-fA\-F]{1,4}){1,5}\|[0\-9a\-fA\-F]{1,4}\:((\:[0\-9a\-fA\-F]{1,4}){1,6})\|\:((\:[0\-9a\-fA\-F]{1,4}){1,7}\|\:))$
                
                    **config**: False
                
                .. attribute:: desired_minimum_tx_interval
                
                    The minimum interval between transmission of BFD control packets that the operator desires. This value is advertised to the peer, however the actual interval used is specified by taking the maximum of desired\-minimum\-tx\-interval and the value of the remote required\-minimum\-receive interval value.  This value is specified as an integer number of microseconds
                
                    **type**: int
                
                        **range:** 0..4294967295
                
                    **config**: False
                
                    **units**: microseconds
                
                .. attribute:: required_minimum_receive
                
                    The minimum interval between received BFD control packets that this system should support. This value is advertised to the remote peer to indicate the maximum frequency (i.e., minimum inter\-packet interval) between BFD control packets that is acceptable to the local system
                
                    **type**: int
                
                        **range:** 0..4294967295
                
                    **config**: False
                
                    **units**: microseconds
                
                .. attribute:: detection_multiplier
                
                    The number of packets that must be missed to declare this session as down. The detection interval for the BFD session is calculated by multiplying the value of the negotiated transmission interval by this value
                
                    **type**: int
                
                        **range:** 1..65535
                
                    **config**: False
                
                .. attribute:: enable_per_member_link
                
                    When this leaf is set to true \- BFD will be enabled on each member interface of the aggregated Ethernet bundle
                
                    **type**: bool
                
                    **config**: False
                
                    **default value**: false
                
                """
                _prefix = 'oc-bfd'
                _revision = '2018-11-21'

                def __init__(self):
                    super().__init__()

                    self.yang_name = "state"
                    self.yang_parent_name = "interface"
                    self.is_top_level_class = False
                    self.has_list_ancestor = True
                    self.ylist_key_names = []
                    self._child_classes = OrderedDict([])
                    self._leafs = OrderedDict([
                        ('id', (YLeaf(YType.str, 'id'), ['str'])),
                        ('enabled', (YLeaf(YType.boolean, 'enabled'), ['bool'])),
                        ('local_address', (YLeaf(YType.str, 'local-address'), ['str','str'])),
                        ('desired_minimum_tx_interval', (YLeaf(YType.uint32, 'desired-minimum-tx-interval'), ['int'])),
                        ('required_minimum_receive', (YLeaf(YType.uint32, 'required-minimum-receive'), ['int'])),
                        ('detection_multiplier', (YLeaf(YType.uint16, 'detection-multiplier'), ['int'])),
                        ('enable_per_member_link', (YLeaf(YType.boolean, 'enable-per-member-link'), ['bool'])),
                    ])
                    self.id = None
                    self.enabled = None
                    self.local_address = None
                    self.desired_minimum_tx_interval = None
                    self.required_minimum_receive = None
                    self.detection_multiplier = None
                    self.enable_per_member_link = None
                    self._segment_path = lambda: "state"
                    self._is_frozen = True

                def __setattr__(self, name, value):
                    self._perform_setattr(Bfd.Interfaces.Interface.State, ['id', 'enabled', 'local_address', 'desired_minimum_tx_interval', 'required_minimum_receive', 'detection_multiplier', 'enable_per_member_link'], name, value)


            class InterfaceRef(_Entity_):
                """
                Reference to an interface or subinterface
                
                .. attribute:: config
                
                    Configured reference to interface / subinterface
                
                    **type**:  :py:class:`Config<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.InterfaceRef.Config>`
                
                .. attribute:: state
                
                    Operational state for interface\-ref
                
                    **type**:  :py:class:`State<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.InterfaceRef.State>`
                
                    **config**: False
                
                """
                _prefix = 'oc-bfd'
                _revision = '2018-11-21'

                def __init__(self):
                    super().__init__()

                    self.yang_name = "interface-ref"
                    self.yang_parent_name = "interface"
                    self.is_top_level_class = False
                    self.has_list_ancestor = True
                    self.ylist_key_names = []
                    self._child_classes = OrderedDict([("config", ("config", Bfd.Interfaces.Interface.InterfaceRef.Config)), ("state", ("state", Bfd.Interfaces.Interface.InterfaceRef.State))])
                    self._leafs = OrderedDict()

                    self.config = Bfd.Interfaces.Interface.InterfaceRef.Config()
                    self.config.parent = self
                    self._children_name_map["config"] = "config"

                    self.state = Bfd.Interfaces.Interface.InterfaceRef.State()
                    self.state.parent = self
                    self._children_name_map["state"] = "state"
                    self._segment_path = lambda: "interface-ref"
                    self._is_frozen = True

                def __setattr__(self, name, value):
                    self._perform_setattr(Bfd.Interfaces.Interface.InterfaceRef, [], name, value)

                class Config(_Entity_):
                    """
                    Configured reference to interface / subinterface
                    
                    .. attribute:: interface
                    
                        Reference to a base interface.  If a reference to a subinterface is required, this leaf must be specified to indicate the base interface
                    
                        **type**: str
                    
                        **refers to**:  :py:class:`name<ydk.models.openconfig.openconfig_interfaces.Interfaces.Interface>`
                    
                    .. attribute:: subinterface
                    
                        Reference to a subinterface \-\- this requires the base interface to be specified using the interface leaf in this container.  If only a reference to a base interface is requuired, this leaf should not be set
                    
                        **type**: int
                    
                            **range:** 0..4294967295
                    
                        **refers to**:  :py:class:`index<ydk.models.openconfig.openconfig_interfaces.Interfaces.Interface.Subinterfaces.Subinterface>`
                    
                    """
                    _prefix = 'oc-bfd'
                    _revision = '2018-11-21'

                    def __init__(self):
                        super().__init__()

                        self.yang_name = "config"
                        self.yang_parent_name = "interface-ref"
                        self.is_top_level_class = False
                        self.has_list_ancestor = True
                        self.ylist_key_names = []
                        self._child_classes = OrderedDict([])
                        self._leafs = OrderedDict([
                            ('interface', (YLeaf(YType.str, 'interface'), ['str'])),
                            ('subinterface', (YLeaf(YType.str, 'subinterface'), ['int'])),
                        ])
                        self.interface = None
                        self.subinterface = None
                        self._segment_path = lambda: "config"
                        self._is_frozen = True

                    def __setattr__(self, name, value):
                        self._perform_setattr(Bfd.Interfaces.Interface.InterfaceRef.Config, ['interface', 'subinterface'], name, value)


                class State(_Entity_):
                    """
                    Operational state for interface\-ref
                    
                    .. attribute:: interface
                    
                        Reference to a base interface.  If a reference to a subinterface is required, this leaf must be specified to indicate the base interface
                    
                        **type**: str
                    
                        **refers to**:  :py:class:`name<ydk.models.openconfig.openconfig_interfaces.Interfaces.Interface>`
                    
                        **config**: False
                    
                    .. attribute:: subinterface
                    
                        Reference to a subinterface \-\- this requires the base interface to be specified using the interface leaf in this container.  If only a reference to a base interface is requuired, this leaf should not be set
                    
                        **type**: int
                    
                            **range:** 0..4294967295
                    
                        **refers to**:  :py:class:`index<ydk.models.openconfig.openconfig_interfaces.Interfaces.Interface.Subinterfaces.Subinterface>`
                    
                        **config**: False
                    
                    """
                    _prefix = 'oc-bfd'
                    _revision = '2018-11-21'

                    def __init__(self):
                        super().__init__()

                        self.yang_name = "state"
                        self.yang_parent_name = "interface-ref"
                        self.is_top_level_class = False
                        self.has_list_ancestor = True
                        self.ylist_key_names = []
                        self._child_classes = OrderedDict([])
                        self._leafs = OrderedDict([
                            ('interface', (YLeaf(YType.str, 'interface'), ['str'])),
                            ('subinterface', (YLeaf(YType.str, 'subinterface'), ['int'])),
                        ])
                        self.interface = None
                        self.subinterface = None
                        self._segment_path = lambda: "state"
                        self._is_frozen = True

                    def __setattr__(self, name, value):
                        self._perform_setattr(Bfd.Interfaces.Interface.InterfaceRef.State, ['interface', 'subinterface'], name, value)



            class MicroBfdSessions(_Entity_):
                """
                Parameters relating to micro\-BFD sessions associated
                with the interface.
                
                .. attribute:: micro_bfd_session
                
                    This list contains configuration and state parameters relating to micro\-BFD session
                
                    **type**: list of    :py:class:`MicroBfdSession<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession>`
                
                """
                _prefix = 'oc-bfd'
                _revision = '2018-11-21'

                def __init__(self):
                    super().__init__()

                    self.yang_name = "micro-bfd-sessions"
                    self.yang_parent_name = "interface"
                    self.is_top_level_class = False
                    self.has_list_ancestor = True
                    self.ylist_key_names = []
                    self._child_classes = OrderedDict([("micro-bfd-session", ("micro_bfd_session", Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession))])
                    self._leafs = OrderedDict()

                    self.micro_bfd_session = YList(self)
                    self._segment_path = lambda: "micro-bfd-sessions"
                    self._is_frozen = True

                def __setattr__(self, name, value):
                    self._perform_setattr(Bfd.Interfaces.Interface.MicroBfdSessions, [], name, value)

                class MicroBfdSession(_Entity_):
                    """
                    This list contains configuration and state parameters
                    relating to micro\-BFD session.
                    
                    .. attribute:: member_interface  (key)
                    
                        A reference to the member interface of the link aggregate
                    
                        **type**: str
                    
                        **refers to**:  :py:class:`member_interface<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.Config>`
                    
                    .. attribute:: config
                    
                        Configuration parameters for the micro\-BFD session
                    
                        **type**:  :py:class:`Config<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.Config>`
                    
                    .. attribute:: state
                    
                        Operational state parameters for the micro\-BFD session
                    
                        **type**:  :py:class:`State<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.State>`
                    
                        **config**: False
                    
                    """
                    _prefix = 'oc-bfd'
                    _revision = '2018-11-21'

                    def __init__(self):
                        super().__init__()

                        self.yang_name = "micro-bfd-session"
                        self.yang_parent_name = "micro-bfd-sessions"
                        self.is_top_level_class = False
                        self.has_list_ancestor = True
                        self.ylist_key_names = ['member_interface']
                        self._child_classes = OrderedDict([("config", ("config", Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.Config)), ("state", ("state", Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.State))])
                        self._leafs = OrderedDict([
                            ('member_interface', (YLeaf(YType.str, 'member-interface'), ['str'])),
                        ])
                        self.member_interface = None

                        self.config = Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.Config()
                        self.config.parent = self
                        self._children_name_map["config"] = "config"

                        self.state = Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.State()
                        self.state.parent = self
                        self._children_name_map["state"] = "state"
                        self._segment_path = lambda: "micro-bfd-session" + "[member-interface='" + str(self.member_interface) + "']"
                        self._is_frozen = True

                    def __setattr__(self, name, value):
                        self._perform_setattr(Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession, ['member_interface'], name, value)

                    class Config(_Entity_):
                        """
                        Configuration parameters for the micro\-BFD session.
                        
                        .. attribute:: local_address
                        
                            The local IP address used by the system for the micro\-BFD session specified
                        
                            **type**: union of the below types:
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])\\.){3}([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])$
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9a\-fA\-F]{1,4}\:){7}[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,7}\:\|([0\-9a\-fA\-F]{1,4}\:){1,6}\:[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,5}(\:[0\-9a\-fA\-F]{1,4}){1,2}\|([0\-9a\-fA\-F]{1,4}\:){1,4}(\:[0\-9a\-fA\-F]{1,4}){1,3}\|([0\-9a\-fA\-F]{1,4}\:){1,3}(\:[0\-9a\-fA\-F]{1,4}){1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,2}(\:[0\-9a\-fA\-F]{1,4}){1,5}\|[0\-9a\-fA\-F]{1,4}\:((\:[0\-9a\-fA\-F]{1,4}){1,6})\|\:((\:[0\-9a\-fA\-F]{1,4}){1,7}\|\:))$
                        
                        .. attribute:: remote_address
                        
                            The remote IP destination that should be used by the system for the micro\-BFD session specified
                        
                            **type**: union of the below types:
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])\\.){3}([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])$
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9a\-fA\-F]{1,4}\:){7}[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,7}\:\|([0\-9a\-fA\-F]{1,4}\:){1,6}\:[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,5}(\:[0\-9a\-fA\-F]{1,4}){1,2}\|([0\-9a\-fA\-F]{1,4}\:){1,4}(\:[0\-9a\-fA\-F]{1,4}){1,3}\|([0\-9a\-fA\-F]{1,4}\:){1,3}(\:[0\-9a\-fA\-F]{1,4}){1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,2}(\:[0\-9a\-fA\-F]{1,4}){1,5}\|[0\-9a\-fA\-F]{1,4}\:((\:[0\-9a\-fA\-F]{1,4}){1,6})\|\:((\:[0\-9a\-fA\-F]{1,4}){1,7}\|\:))$
                        
                        .. attribute:: member_interface
                        
                            Reference to a member link of the aggregate interface being described
                        
                            **type**: str
                        
                            **refers to**:  :py:class:`name<ydk.models.openconfig.openconfig_interfaces.Interfaces.Interface.Config>`
                        
                        """
                        _prefix = 'oc-bfd'
                        _revision = '2018-11-21'

                        def __init__(self):
                            super().__init__()

                            self.yang_name = "config"
                            self.yang_parent_name = "micro-bfd-session"
                            self.is_top_level_class = False
                            self.has_list_ancestor = True
                            self.ylist_key_names = []
                            self._child_classes = OrderedDict([])
                            self._leafs = OrderedDict([
                                ('local_address', (YLeaf(YType.str, 'local-address'), ['str','str'])),
                                ('remote_address', (YLeaf(YType.str, 'remote-address'), ['str','str'])),
                                ('member_interface', (YLeaf(YType.str, 'member-interface'), ['str'])),
                            ])
                            self.local_address = None
                            self.remote_address = None
                            self.member_interface = None
                            self._segment_path = lambda: "config"
                            self._is_frozen = True

                        def __setattr__(self, name, value):
                            self._perform_setattr(Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.Config, ['local_address', 'remote_address', 'member_interface'], name, value)


                    class State(_Entity_):
                        """
                        Operational state parameters for the micro\-BFD session.
                        
                        .. attribute:: local_address
                        
                            The local IP address used by the system for the micro\-BFD session specified
                        
                            **type**: union of the below types:
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])\\.){3}([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])$
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9a\-fA\-F]{1,4}\:){7}[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,7}\:\|([0\-9a\-fA\-F]{1,4}\:){1,6}\:[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,5}(\:[0\-9a\-fA\-F]{1,4}){1,2}\|([0\-9a\-fA\-F]{1,4}\:){1,4}(\:[0\-9a\-fA\-F]{1,4}){1,3}\|([0\-9a\-fA\-F]{1,4}\:){1,3}(\:[0\-9a\-fA\-F]{1,4}){1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,2}(\:[0\-9a\-fA\-F]{1,4}){1,5}\|[0\-9a\-fA\-F]{1,4}\:((\:[0\-9a\-fA\-F]{1,4}){1,6})\|\:((\:[0\-9a\-fA\-F]{1,4}){1,7}\|\:))$
                        
                            **config**: False
                        
                        .. attribute:: remote_address
                        
                            The remote IP destination that should be used by the system for the micro\-BFD session specified
                        
                            **type**: union of the below types:
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])\\.){3}([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])$
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9a\-fA\-F]{1,4}\:){7}[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,7}\:\|([0\-9a\-fA\-F]{1,4}\:){1,6}\:[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,5}(\:[0\-9a\-fA\-F]{1,4}){1,2}\|([0\-9a\-fA\-F]{1,4}\:){1,4}(\:[0\-9a\-fA\-F]{1,4}){1,3}\|([0\-9a\-fA\-F]{1,4}\:){1,3}(\:[0\-9a\-fA\-F]{1,4}){1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,2}(\:[0\-9a\-fA\-F]{1,4}){1,5}\|[0\-9a\-fA\-F]{1,4}\:((\:[0\-9a\-fA\-F]{1,4}){1,6})\|\:((\:[0\-9a\-fA\-F]{1,4}){1,7}\|\:))$
                        
                            **config**: False
                        
                        .. attribute:: member_interface
                        
                            Reference to a member link of the aggregate interface being described
                        
                            **type**: str
                        
                            **refers to**:  :py:class:`name<ydk.models.openconfig.openconfig_interfaces.Interfaces.Interface.Config>`
                        
                            **config**: False
                        
                        .. attribute:: session_state
                        
                            The state of the BFD session perceived by the local system
                        
                            **type**:  :py:class:`BfdSessionState<ydk.models.openconfig.openconfig_bfd.BfdSessionState>`
                        
                            **config**: False
                        
                        .. attribute:: remote_session_state
                        
                            The reported state of the BFD session according to the remote system. This state reflects the last state reported in a BFD control packet
                        
                            **type**:  :py:class:`BfdSessionState<ydk.models.openconfig.openconfig_bfd.BfdSessionState>`
                        
                            **config**: False
                        
                        .. attribute:: last_failure_time
                        
                            The time of the last transition of the BFD session out of the UP state, expressed as the number of nanoseconds since the Unix epoch
                        
                            **type**: int
                        
                                **range:** 0..18446744073709551615
                        
                            **config**: False
                        
                        .. attribute:: failure_transitions
                        
                            The number of times that the BFD session has transitioned out of the UP state
                        
                            **type**: int
                        
                                **range:** 0..18446744073709551615
                        
                            **config**: False
                        
                        .. attribute:: local_discriminator
                        
                            A unique identifier used by the local system to identify this BFD session
                        
                            **type**: str
                        
                            **config**: False
                        
                        .. attribute:: remote_discriminator
                        
                            A unique identified used by the remote system to identify this BFD session
                        
                            **type**: str
                        
                            **config**: False
                        
                        .. attribute:: local_diagnostic_code
                        
                            The local BFD diagnostic code indicating the most recent reason for failure of this BFD session
                        
                            **type**:  :py:class:`BfdDiagnosticCode<ydk.models.openconfig.openconfig_bfd.BfdDiagnosticCode>`
                        
                            **config**: False
                        
                        .. attribute:: remote_diagnostic_code
                        
                            The remote BFD diagnostic code indicating the remote system's reason for failure of the BFD session
                        
                            **type**:  :py:class:`BfdDiagnosticCode<ydk.models.openconfig.openconfig_bfd.BfdDiagnosticCode>`
                        
                            **config**: False
                        
                        .. attribute:: remote_minimum_receive_interval
                        
                            The value of the minimum receive interval that was specified in the most recent BFD control packet received from the peer
                        
                            **type**: int
                        
                                **range:** 0..4294967295
                        
                            **config**: False
                        
                        .. attribute:: demand_mode_requested
                        
                            This leaf is set to true when the remote system has requested demand mode be run for this session
                        
                            **type**: bool
                        
                            **config**: False
                        
                        .. attribute:: remote_authentication_enabled
                        
                            This leaf is set to true when the remote system has specified that authentication is present for the BFD session
                        
                            **type**: bool
                        
                            **config**: False
                        
                        .. attribute:: remote_control_plane_independent
                        
                            This leaf is set to true when the remote system has specified that the hardware implementing this BFD session is independent of the control plane's liveliness
                        
                            **type**: bool
                        
                            **config**: False
                        
                        .. attribute:: async_
                        
                            Operational state parameters specifically relating to asynchronous mode of BFD
                        
                            **type**:  :py:class:`Async<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.State.Async>`
                        
                            **config**: False
                        
                        """
                        _prefix = 'oc-bfd'
                        _revision = '2018-11-21'

                        def __init__(self):
                            super().__init__()

                            self.yang_name = "state"
                            self.yang_parent_name = "micro-bfd-session"
                            self.is_top_level_class = False
                            self.has_list_ancestor = True
                            self.ylist_key_names = []
                            self._child_classes = OrderedDict([("async", ("async_", Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.State.Async))])
                            self._leafs = OrderedDict([
                                ('local_address', (YLeaf(YType.str, 'local-address'), ['str','str'])),
                                ('remote_address', (YLeaf(YType.str, 'remote-address'), ['str','str'])),
                                ('member_interface', (YLeaf(YType.str, 'member-interface'), ['str'])),
                                ('session_state', (YLeaf(YType.enumeration, 'session-state'), [('ydk.models.openconfig.openconfig_bfd', 'BfdSessionState', '')])),
                                ('remote_session_state', (YLeaf(YType.enumeration, 'remote-session-state'), [('ydk.models.openconfig.openconfig_bfd', 'BfdSessionState', '')])),
                                ('last_failure_time', (YLeaf(YType.uint64, 'last-failure-time'), ['int'])),
                                ('failure_transitions', (YLeaf(YType.uint64, 'failure-transitions'), ['int'])),
                                ('local_discriminator', (YLeaf(YType.str, 'local-discriminator'), ['str'])),
                                ('remote_discriminator', (YLeaf(YType.str, 'remote-discriminator'), ['str'])),
                                ('local_diagnostic_code', (YLeaf(YType.enumeration, 'local-diagnostic-code'), [('ydk.models.openconfig.openconfig_bfd', 'BfdDiagnosticCode', '')])),
                                ('remote_diagnostic_code', (YLeaf(YType.enumeration, 'remote-diagnostic-code'), [('ydk.models.openconfig.openconfig_bfd', 'BfdDiagnosticCode', '')])),
                                ('remote_minimum_receive_interval', (YLeaf(YType.uint32, 'remote-minimum-receive-interval'), ['int'])),
                                ('demand_mode_requested', (YLeaf(YType.boolean, 'demand-mode-requested'), ['bool'])),
                                ('remote_authentication_enabled', (YLeaf(YType.boolean, 'remote-authentication-enabled'), ['bool'])),
                                ('remote_control_plane_independent', (YLeaf(YType.boolean, 'remote-control-plane-independent'), ['bool'])),
                            ])
                            self.local_address = None
                            self.remote_address = None
                            self.member_interface = None
                            self.session_state = None
                            self.remote_session_state = None
                            self.last_failure_time = None
                            self.failure_transitions = None
                            self.local_discriminator = None
                            self.remote_discriminator = None
                            self.local_diagnostic_code = None
                            self.remote_diagnostic_code = None
                            self.remote_minimum_receive_interval = None
                            self.demand_mode_requested = None
                            self.remote_authentication_enabled = None
                            self.remote_control_plane_independent = None

                            self.async_ = Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.State.Async()
                            self.async_.parent = self
                            self._children_name_map["async_"] = "async"
                            self._segment_path = lambda: "state"
                            self._is_frozen = True

                        def __setattr__(self, name, value):
                            self._perform_setattr(Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.State, ['local_address', 'remote_address', 'member_interface', 'session_state', 'remote_session_state', 'last_failure_time', 'failure_transitions', 'local_discriminator', 'remote_discriminator', 'local_diagnostic_code', 'remote_diagnostic_code', 'remote_minimum_receive_interval', 'demand_mode_requested', 'remote_authentication_enabled', 'remote_control_plane_independent'], name, value)

                        class Async(_Entity_):
                            """
                            Operational state parameters specifically relating to
                            asynchronous mode of BFD.
                            
                            .. attribute:: last_packet_transmitted
                            
                                The date and time at which the last BFD packet was transmitted for this session, expressed as the number of nanoseconds since the Unix Epoch (January 1, 1970, 00\:00 UTC)
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: last_packet_received
                            
                                The date and time at which the last BFD packet was received for this session, expressed as the number of nanoseconds since the Unix Epoch (January 1, 1970, 00\:00 UTC)
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: transmitted_packets
                            
                                The number of packets that have been transmitted by the local system
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: received_packets
                            
                                The number of packets that have been received by the local system from the remote neighbour
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: up_transitions
                            
                                The number of times that the adjacency with the neighbor has transitioned into the up state
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            """
                            _prefix = 'oc-bfd'
                            _revision = '2018-11-21'

                            def __init__(self):
                                super().__init__()

                                self.yang_name = "async"
                                self.yang_parent_name = "state"
                                self.is_top_level_class = False
                                self.has_list_ancestor = True
                                self.ylist_key_names = []
                                self._child_classes = OrderedDict([])
                                self._leafs = OrderedDict([
                                    ('last_packet_transmitted', (YLeaf(YType.uint64, 'last-packet-transmitted'), ['int'])),
                                    ('last_packet_received', (YLeaf(YType.uint64, 'last-packet-received'), ['int'])),
                                    ('transmitted_packets', (YLeaf(YType.uint64, 'transmitted-packets'), ['int'])),
                                    ('received_packets', (YLeaf(YType.uint64, 'received-packets'), ['int'])),
                                    ('up_transitions', (YLeaf(YType.uint64, 'up-transitions'), ['int'])),
                                ])
                                self.last_packet_transmitted = None
                                self.last_packet_received = None
                                self.transmitted_packets = None
                                self.received_packets = None
                                self.up_transitions = None
                                self._segment_path = lambda: "async"
                                self._is_frozen = True

                            def __setattr__(self, name, value):
                                self._perform_setattr(Bfd.Interfaces.Interface.MicroBfdSessions.MicroBfdSession.State.Async, ['last_packet_transmitted', 'last_packet_received', 'transmitted_packets', 'received_packets', 'up_transitions'], name, value)





            class Peers(_Entity_):
                """
                Parameters relating to the BFD peers which are seen
                over this interface.
                
                .. attribute:: peer
                
                    Parameters relating to the BFD peer specified by the remote address
                
                    **type**: list of    :py:class:`Peer<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.Peers.Peer>`
                
                    **config**: False
                
                """
                _prefix = 'oc-bfd'
                _revision = '2018-11-21'

                def __init__(self):
                    super().__init__()

                    self.yang_name = "peers"
                    self.yang_parent_name = "interface"
                    self.is_top_level_class = False
                    self.has_list_ancestor = True
                    self.ylist_key_names = []
                    self._child_classes = OrderedDict([("peer", ("peer", Bfd.Interfaces.Interface.Peers.Peer))])
                    self._leafs = OrderedDict()

                    self.peer = YList(self)
                    self._segment_path = lambda: "peers"
                    self._is_frozen = True

                def __setattr__(self, name, value):
                    self._perform_setattr(Bfd.Interfaces.Interface.Peers, [], name, value)

                class Peer(_Entity_):
                    """
                    Parameters relating to the BFD peer specified by the
                    remote address.
                    
                    .. attribute:: local_discriminator  (key)
                    
                        The local discriminator, which is unique for the session on the system
                    
                        **type**: str
                    
                        **refers to**:  :py:class:`local_discriminator<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.Peers.Peer.State>`
                    
                        **config**: False
                    
                    .. attribute:: state
                    
                        Operational state parameters for the BFD session
                    
                        **type**:  :py:class:`State<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.Peers.Peer.State>`
                    
                        **config**: False
                    
                    """
                    _prefix = 'oc-bfd'
                    _revision = '2018-11-21'

                    def __init__(self):
                        super().__init__()

                        self.yang_name = "peer"
                        self.yang_parent_name = "peers"
                        self.is_top_level_class = False
                        self.has_list_ancestor = True
                        self.ylist_key_names = ['local_discriminator']
                        self._child_classes = OrderedDict([("state", ("state", Bfd.Interfaces.Interface.Peers.Peer.State))])
                        self._leafs = OrderedDict([
                            ('local_discriminator', (YLeaf(YType.str, 'local-discriminator'), ['str'])),
                        ])
                        self.local_discriminator = None

                        self.state = Bfd.Interfaces.Interface.Peers.Peer.State()
                        self.state.parent = self
                        self._children_name_map["state"] = "state"
                        self._segment_path = lambda: "peer" + "[local-discriminator='" + str(self.local_discriminator) + "']"
                        self._is_frozen = True

                    def __setattr__(self, name, value):
                        self._perform_setattr(Bfd.Interfaces.Interface.Peers.Peer, ['local_discriminator'], name, value)

                    class State(_Entity_):
                        """
                        Operational state parameters for the BFD session.
                        
                        .. attribute:: local_address
                        
                            The IP address used by the local system for this BFD session
                        
                            **type**: union of the below types:
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])\\.){3}([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])$
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9a\-fA\-F]{1,4}\:){7}[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,7}\:\|([0\-9a\-fA\-F]{1,4}\:){1,6}\:[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,5}(\:[0\-9a\-fA\-F]{1,4}){1,2}\|([0\-9a\-fA\-F]{1,4}\:){1,4}(\:[0\-9a\-fA\-F]{1,4}){1,3}\|([0\-9a\-fA\-F]{1,4}\:){1,3}(\:[0\-9a\-fA\-F]{1,4}){1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,2}(\:[0\-9a\-fA\-F]{1,4}){1,5}\|[0\-9a\-fA\-F]{1,4}\:((\:[0\-9a\-fA\-F]{1,4}){1,6})\|\:((\:[0\-9a\-fA\-F]{1,4}){1,7}\|\:))$
                        
                            **config**: False
                        
                        .. attribute:: remote_address
                        
                            The IP address used by the remote system for this BFD session
                        
                            **type**: union of the below types:
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])\\.){3}([0\-9]\|[1\-9][0\-9]\|1[0\-9][0\-9]\|2[0\-4][0\-9]\|25[0\-5])$
                        
                                **type**: str
                        
                                    **pattern:** ^(([0\-9a\-fA\-F]{1,4}\:){7}[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,7}\:\|([0\-9a\-fA\-F]{1,4}\:){1,6}\:[0\-9a\-fA\-F]{1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,5}(\:[0\-9a\-fA\-F]{1,4}){1,2}\|([0\-9a\-fA\-F]{1,4}\:){1,4}(\:[0\-9a\-fA\-F]{1,4}){1,3}\|([0\-9a\-fA\-F]{1,4}\:){1,3}(\:[0\-9a\-fA\-F]{1,4}){1,4}\|([0\-9a\-fA\-F]{1,4}\:){1,2}(\:[0\-9a\-fA\-F]{1,4}){1,5}\|[0\-9a\-fA\-F]{1,4}\:((\:[0\-9a\-fA\-F]{1,4}){1,6})\|\:((\:[0\-9a\-fA\-F]{1,4}){1,7}\|\:))$
                        
                            **config**: False
                        
                        .. attribute:: subscribed_protocols
                        
                            Indicates the set of protocols that currently use this BFD session for liveliness detection
                        
                            **type**: list of   :py:class:`INSTALLPROTOCOLTYPE<ydk.models.openconfig.openconfig_policy_types.INSTALLPROTOCOLTYPE>`
                        
                            **config**: False
                        
                        .. attribute:: session_state
                        
                            The state of the BFD session perceived by the local system
                        
                            **type**:  :py:class:`BfdSessionState<ydk.models.openconfig.openconfig_bfd.BfdSessionState>`
                        
                            **config**: False
                        
                        .. attribute:: remote_session_state
                        
                            The reported state of the BFD session according to the remote system. This state reflects the last state reported in a BFD control packet
                        
                            **type**:  :py:class:`BfdSessionState<ydk.models.openconfig.openconfig_bfd.BfdSessionState>`
                        
                            **config**: False
                        
                        .. attribute:: last_failure_time
                        
                            The time of the last transition of the BFD session out of the UP state, expressed as the number of nanoseconds since the Unix epoch
                        
                            **type**: int
                        
                                **range:** 0..18446744073709551615
                        
                            **config**: False
                        
                        .. attribute:: failure_transitions
                        
                            The number of times that the BFD session has transitioned out of the UP state
                        
                            **type**: int
                        
                                **range:** 0..18446744073709551615
                        
                            **config**: False
                        
                        .. attribute:: local_discriminator
                        
                            A unique identifier used by the local system to identify this BFD session
                        
                            **type**: str
                        
                            **config**: False
                        
                        .. attribute:: remote_discriminator
                        
                            A unique identified used by the remote system to identify this BFD session
                        
                            **type**: str
                        
                            **config**: False
                        
                        .. attribute:: local_diagnostic_code
                        
                            The local BFD diagnostic code indicating the most recent reason for failure of this BFD session
                        
                            **type**:  :py:class:`BfdDiagnosticCode<ydk.models.openconfig.openconfig_bfd.BfdDiagnosticCode>`
                        
                            **config**: False
                        
                        .. attribute:: remote_diagnostic_code
                        
                            The remote BFD diagnostic code indicating the remote system's reason for failure of the BFD session
                        
                            **type**:  :py:class:`BfdDiagnosticCode<ydk.models.openconfig.openconfig_bfd.BfdDiagnosticCode>`
                        
                            **config**: False
                        
                        .. attribute:: remote_minimum_receive_interval
                        
                            The value of the minimum receive interval that was specified in the most recent BFD control packet received from the peer
                        
                            **type**: int
                        
                                **range:** 0..4294967295
                        
                            **config**: False
                        
                        .. attribute:: demand_mode_requested
                        
                            This leaf is set to true when the remote system has requested demand mode be run for this session
                        
                            **type**: bool
                        
                            **config**: False
                        
                        .. attribute:: remote_authentication_enabled
                        
                            This leaf is set to true when the remote system has specified that authentication is present for the BFD session
                        
                            **type**: bool
                        
                            **config**: False
                        
                        .. attribute:: remote_control_plane_independent
                        
                            This leaf is set to true when the remote system has specified that the hardware implementing this BFD session is independent of the control plane's liveliness
                        
                            **type**: bool
                        
                            **config**: False
                        
                        .. attribute:: echo
                        
                            Operational state parameters specifically relating to the echo mode of BFD
                        
                            **type**:  :py:class:`Echo<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.Peers.Peer.State.Echo>`
                        
                            **config**: False
                        
                        .. attribute:: async_
                        
                            Operational state parameters specifically relating to asynchronous mode of BFD
                        
                            **type**:  :py:class:`Async<ydk.models.openconfig.openconfig_bfd.Bfd.Interfaces.Interface.Peers.Peer.State.Async>`
                        
                            **config**: False
                        
                        """
                        _prefix = 'oc-bfd'
                        _revision = '2018-11-21'

                        def __init__(self):
                            super().__init__()

                            self.yang_name = "state"
                            self.yang_parent_name = "peer"
                            self.is_top_level_class = False
                            self.has_list_ancestor = True
                            self.ylist_key_names = []
                            self._child_classes = OrderedDict([("echo", ("echo", Bfd.Interfaces.Interface.Peers.Peer.State.Echo)), ("async", ("async_", Bfd.Interfaces.Interface.Peers.Peer.State.Async))])
                            self._leafs = OrderedDict([
                                ('local_address', (YLeaf(YType.str, 'local-address'), ['str','str'])),
                                ('remote_address', (YLeaf(YType.str, 'remote-address'), ['str','str'])),
                                ('subscribed_protocols', (YLeafList(YType.identityref, 'subscribed-protocols'), [('ydk.models.openconfig.openconfig_policy_types', 'INSTALLPROTOCOLTYPE')])),
                                ('session_state', (YLeaf(YType.enumeration, 'session-state'), [('ydk.models.openconfig.openconfig_bfd', 'BfdSessionState', '')])),
                                ('remote_session_state', (YLeaf(YType.enumeration, 'remote-session-state'), [('ydk.models.openconfig.openconfig_bfd', 'BfdSessionState', '')])),
                                ('last_failure_time', (YLeaf(YType.uint64, 'last-failure-time'), ['int'])),
                                ('failure_transitions', (YLeaf(YType.uint64, 'failure-transitions'), ['int'])),
                                ('local_discriminator', (YLeaf(YType.str, 'local-discriminator'), ['str'])),
                                ('remote_discriminator', (YLeaf(YType.str, 'remote-discriminator'), ['str'])),
                                ('local_diagnostic_code', (YLeaf(YType.enumeration, 'local-diagnostic-code'), [('ydk.models.openconfig.openconfig_bfd', 'BfdDiagnosticCode', '')])),
                                ('remote_diagnostic_code', (YLeaf(YType.enumeration, 'remote-diagnostic-code'), [('ydk.models.openconfig.openconfig_bfd', 'BfdDiagnosticCode', '')])),
                                ('remote_minimum_receive_interval', (YLeaf(YType.uint32, 'remote-minimum-receive-interval'), ['int'])),
                                ('demand_mode_requested', (YLeaf(YType.boolean, 'demand-mode-requested'), ['bool'])),
                                ('remote_authentication_enabled', (YLeaf(YType.boolean, 'remote-authentication-enabled'), ['bool'])),
                                ('remote_control_plane_independent', (YLeaf(YType.boolean, 'remote-control-plane-independent'), ['bool'])),
                            ])
                            self.local_address = None
                            self.remote_address = None
                            self.subscribed_protocols = []
                            self.session_state = None
                            self.remote_session_state = None
                            self.last_failure_time = None
                            self.failure_transitions = None
                            self.local_discriminator = None
                            self.remote_discriminator = None
                            self.local_diagnostic_code = None
                            self.remote_diagnostic_code = None
                            self.remote_minimum_receive_interval = None
                            self.demand_mode_requested = None
                            self.remote_authentication_enabled = None
                            self.remote_control_plane_independent = None

                            self.echo = Bfd.Interfaces.Interface.Peers.Peer.State.Echo()
                            self.echo.parent = self
                            self._children_name_map["echo"] = "echo"

                            self.async_ = Bfd.Interfaces.Interface.Peers.Peer.State.Async()
                            self.async_.parent = self
                            self._children_name_map["async_"] = "async"
                            self._segment_path = lambda: "state"
                            self._is_frozen = True

                        def __setattr__(self, name, value):
                            self._perform_setattr(Bfd.Interfaces.Interface.Peers.Peer.State, ['local_address', 'remote_address', 'subscribed_protocols', 'session_state', 'remote_session_state', 'last_failure_time', 'failure_transitions', 'local_discriminator', 'remote_discriminator', 'local_diagnostic_code', 'remote_diagnostic_code', 'remote_minimum_receive_interval', 'demand_mode_requested', 'remote_authentication_enabled', 'remote_control_plane_independent'], name, value)

                        class Echo(_Entity_):
                            """
                            Operational state parameters specifically relating to the
                            echo mode of BFD.
                            
                            .. attribute:: active
                            
                                This leaf is set to true when echo mode is running between the local and remote system. When it is set to false, solely asynchronous mode is active
                            
                                **type**: bool
                            
                                **config**: False
                            
                            .. attribute:: last_packet_transmitted
                            
                                The date and time at which the last BFD packet was transmitted for this session, expressed as the number of nanoseconds since the Unix Epoch (January 1, 1970, 00\:00 UTC)
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: last_packet_received
                            
                                The date and time at which the last BFD packet was received for this session, expressed as the number of nanoseconds since the Unix Epoch (January 1, 1970, 00\:00 UTC)
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: transmitted_packets
                            
                                The number of packets that have been transmitted by the local system
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: received_packets
                            
                                The number of packets that have been received by the local system from the remote neighbour
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: up_transitions
                            
                                The number of times that the adjacency with the neighbor has transitioned into the up state
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            """
                            _prefix = 'oc-bfd'
                            _revision = '2018-11-21'

                            def __init__(self):
                                super().__init__()

                                self.yang_name = "echo"
                                self.yang_parent_name = "state"
                                self.is_top_level_class = False
                                self.has_list_ancestor = True
                                self.ylist_key_names = []
                                self._child_classes = OrderedDict([])
                                self._leafs = OrderedDict([
                                    ('active', (YLeaf(YType.boolean, 'active'), ['bool'])),
                                    ('last_packet_transmitted', (YLeaf(YType.uint64, 'last-packet-transmitted'), ['int'])),
                                    ('last_packet_received', (YLeaf(YType.uint64, 'last-packet-received'), ['int'])),
                                    ('transmitted_packets', (YLeaf(YType.uint64, 'transmitted-packets'), ['int'])),
                                    ('received_packets', (YLeaf(YType.uint64, 'received-packets'), ['int'])),
                                    ('up_transitions', (YLeaf(YType.uint64, 'up-transitions'), ['int'])),
                                ])
                                self.active = None
                                self.last_packet_transmitted = None
                                self.last_packet_received = None
                                self.transmitted_packets = None
                                self.received_packets = None
                                self.up_transitions = None
                                self._segment_path = lambda: "echo"
                                self._is_frozen = True

                            def __setattr__(self, name, value):
                                self._perform_setattr(Bfd.Interfaces.Interface.Peers.Peer.State.Echo, ['active', 'last_packet_transmitted', 'last_packet_received', 'transmitted_packets', 'received_packets', 'up_transitions'], name, value)


                        class Async(_Entity_):
                            """
                            Operational state parameters specifically relating to
                            asynchronous mode of BFD.
                            
                            .. attribute:: last_packet_transmitted
                            
                                The date and time at which the last BFD packet was transmitted for this session, expressed as the number of nanoseconds since the Unix Epoch (January 1, 1970, 00\:00 UTC)
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: last_packet_received
                            
                                The date and time at which the last BFD packet was received for this session, expressed as the number of nanoseconds since the Unix Epoch (January 1, 1970, 00\:00 UTC)
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: transmitted_packets
                            
                                The number of packets that have been transmitted by the local system
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: received_packets
                            
                                The number of packets that have been received by the local system from the remote neighbour
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            .. attribute:: up_transitions
                            
                                The number of times that the adjacency with the neighbor has transitioned into the up state
                            
                                **type**: int
                            
                                    **range:** 0..18446744073709551615
                            
                                **config**: False
                            
                            """
                            _prefix = 'oc-bfd'
                            _revision = '2018-11-21'

                            def __init__(self):
                                super().__init__()

                                self.yang_name = "async"
                                self.yang_parent_name = "state"
                                self.is_top_level_class = False
                                self.has_list_ancestor = True
                                self.ylist_key_names = []
                                self._child_classes = OrderedDict([])
                                self._leafs = OrderedDict([
                                    ('last_packet_transmitted', (YLeaf(YType.uint64, 'last-packet-transmitted'), ['int'])),
                                    ('last_packet_received', (YLeaf(YType.uint64, 'last-packet-received'), ['int'])),
                                    ('transmitted_packets', (YLeaf(YType.uint64, 'transmitted-packets'), ['int'])),
                                    ('received_packets', (YLeaf(YType.uint64, 'received-packets'), ['int'])),
                                    ('up_transitions', (YLeaf(YType.uint64, 'up-transitions'), ['int'])),
                                ])
                                self.last_packet_transmitted = None
                                self.last_packet_received = None
                                self.transmitted_packets = None
                                self.received_packets = None
                                self.up_transitions = None
                                self._segment_path = lambda: "async"
                                self._is_frozen = True

                            def __setattr__(self, name, value):
                                self._perform_setattr(Bfd.Interfaces.Interface.Peers.Peer.State.Async, ['last_packet_transmitted', 'last_packet_received', 'transmitted_packets', 'received_packets', 'up_transitions'], name, value)







    def clone_ptr(self):
        self._top_entity = Bfd()
        return self._top_entity



